"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ValidateImportFromUrlUseCase = void 0;
var _types = require("../../../types");
var _error = require("@webiny/error");
var _getImportExportFileType = require("../../../tasks/utils/helpers/getImportExportFileType");
var _parseImportUrlData = require("../../utils/parseImportUrlData");
var _makeSureModelsAreIdentical = require("../../utils/makeSureModelsAreIdentical");
class ValidateImportFromUrlUseCase {
  constructor(params) {
    this.getModel = params.getModel;
    this.getModelToAstConverter = params.getModelToAstConverter;
  }
  async execute(params) {
    const {
      data
    } = params;
    const {
      model: validatedModel,
      files
    } = (0, _parseImportUrlData.parseImportUrlData)(data);
    /**
     * There must be at least one file in the data.
     */
    if (files.length === 0) {
      throw new _error.WebinyError("No files found in the provided data.", "NO_FILES_FOUND");
    }
    /**
     * Next step of the validation is to verify that, at least, one entries type file exists in the data.
     */
    const entries = files.find(file => {
      return file.type === _types.CmsImportExportFileType.ENTRIES;
    });
    if (!entries) {
      throw new _error.WebinyError("No entries file found in the provided data.", "NO_ENTRIES_FILE");
    }
    let model;
    try {
      model = await this.getModel(validatedModel.modelId);
    } catch (ex) {
      if (ex.code !== "NOT_FOUND") {
        throw ex;
      }
      throw new _error.WebinyError(`Model provided in the JSON data, "${validatedModel.modelId}", not found.`, "MODEL_NOT_FOUND", {
        modelId: validatedModel.modelId
      });
    }
    (0, _makeSureModelsAreIdentical.makeSureModelsAreIdentical)({
      getModelToAstConverter: this.getModelToAstConverter,
      model,
      target: validatedModel
    });
    return {
      model,
      files: files.reduce((collection, file) => {
        const result = (0, _getImportExportFileType.getImportExportFileType)(file.head);
        if (result.error) {
          file.error = {
            message: "File type not supported.",
            code: "FILE_TYPE_NOT_SUPPORTED",
            data: {
              type: result.type,
              pathname: result.pathname
            }
          };
          collection.push(file);
          return collection;
        }
        collection.push(file);
        return collection;
      }, [])
    };
  }
}
exports.ValidateImportFromUrlUseCase = ValidateImportFromUrlUseCase;

//# sourceMappingURL=ValidateImportFromUrlUseCase.js.map