"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ExportContentAssets = void 0;
var _cmsEntryFetcher = require("../../utils/cmsEntryFetcher");
var _entryAssets = require("../../utils/entryAssets");
var _cmsAssetsZipper = require("../../utils/cmsAssetsZipper");
var _fileFetcher = require("../../utils/fileFetcher");
var _utils = require("@webiny/tasks/utils");
var _getBucket = require("../../utils/helpers/getBucket");
var _s3Client = require("../../utils/helpers/s3Client");
var _UniqueResolver = require("../../utils/uniqueResolver/UniqueResolver");
var _constants = require("../../constants");
const getFilename = input => {
  const current = [input.entryAfter, input.fileAfter].filter(item => item !== undefined).join("-");
  return `${input.prefix}/assets${current ? `-${current}` : ""}.${_constants.WEBINY_EXPORT_ASSETS_EXTENSION}`;
};
class ExportContentAssets {
  constructor(params) {
    this.createCmsAssetsZipper = params.createCmsAssetsZipper;
  }
  async run(params) {
    const {
      response,
      context,
      input,
      isCloseToTimeout,
      isAborted
    } = params;
    let model;
    try {
      model = await context.cms.getModel(input.modelId);
    } catch (ex) {
      return response.error({
        message: `Could not fetch entry manager for model "${input.modelId}".`,
        code: "MODEL_NOT_FOUND",
        data: {
          error: (0, _utils.getErrorProperties)(ex)
        }
      });
    }
    const traverser = await context.cms.getEntryTraverser(model.modelId);
    const entryFetcher = (0, _cmsEntryFetcher.createCmsEntryFetcher)(async after => {
      const input = {
        where: params.input.where,
        limit: params.input.limit || 10000,
        sort: params.input.sort,
        after
      };
      const [items, meta] = await context.cms.listLatestEntries(model, input);
      return {
        items,
        meta
      };
    });
    const fileFetcher = new _fileFetcher.FileFetcher({
      client: (0, _s3Client.createS3Client)(),
      bucket: (0, _getBucket.getBucket)()
    });
    const filename = getFilename(input);
    const zipper = this.createCmsAssetsZipper({
      filename,
      fileFetcher,
      entryFetcher,
      createEntryAssets: () => {
        return new _entryAssets.EntryAssets({
          traverser,
          uniqueResolver: new _UniqueResolver.UniqueResolver()
        });
      },
      createEntryAssetsResolver: () => {
        return new _entryAssets.EntryAssetsResolver({
          fetchFiles: async params => {
            const [items, meta] = await context.fileManager.listFiles(params);
            return {
              items,
              meta
            };
          }
        });
      }
    });
    let result;
    try {
      result = await zipper.execute({
        fileAfter: input.fileAfter,
        entryAfter: input.entryAfter,
        isAborted() {
          return isAborted();
        },
        isCloseToTimeout(seconds) {
          return isCloseToTimeout(seconds);
        }
      });
    } catch (ex) {
      return response.error(ex);
    }
    const files = Array.isArray(input.files) ? input.files : [];
    /**
     * Zipper is done, but there is no result?
     * We will output existing input files.
     */
    if (result instanceof _cmsAssetsZipper.CmsAssetsZipperExecuteDoneWithoutResult) {
      return response.done({
        files
      });
    }
    /**
     * Zipper is done and there is a result?
     * We will output existing input files and the new file.
     */
    //
    else if (result instanceof _cmsAssetsZipper.CmsAssetsZipperExecuteDoneResult) {
      return response.done({
        files: files.concat([{
          key: result.key,
          checksum: result.checksum
        }])
      });
    }
    /**
     * Zipper is not done and there is no result?
     * Let's continue with the next iteration.
     */
    //
    else if (result instanceof _cmsAssetsZipper.CmsAssetsZipperExecuteContinueWithoutResult) {
      return response.continue({
        ...input,
        fileAfter: result.fileCursor,
        entryAfter: result.entryCursor
      });
    }
    /**
     * Zipper is not done and there is a result?
     * Let's merge the existing files with the new file and continue with the next iteration.
     */
    //
    else if (result instanceof _cmsAssetsZipper.CmsAssetsZipperExecuteContinueResult) {
      return response.continue({
        ...input,
        fileAfter: result.fileCursor,
        entryAfter: result.entryCursor,
        files: files.concat([{
          key: result.key,
          checksum: result.checksum
        }])
      });
    }
    return response.error({
      message: "Unknown zipper result.",
      code: "UNKNOWN_ZIPPER_RESULT",
      data: {
        type: typeof result,
        constructor: result?.constructor?.name || "unknown constructor",
        result
      }
    });
  }
}
exports.ExportContentAssets = ExportContentAssets;

//# sourceMappingURL=ExportContentAssets.js.map