"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CmsEntryZipper = void 0;
var _CmsEntryZipperExecuteContinueResult = require("./CmsEntryZipperExecuteContinueResult");
var _CmsEntryZipperExecuteDoneResult = require("./CmsEntryZipperExecuteDoneResult");
var _sanitize = require("@webiny/api-headless-cms/export/crud/sanitize");
var _exportPath = require("../helpers/exportPath");
var _cleanChecksum = require("../helpers/cleanChecksum");
var _constants = require("../../constants");
const createBufferData = params => {
  const {
    items,
    meta,
    after
  } = params;
  return Buffer.from(JSON.stringify({
    items: items.map(item => {
      /**
       * We will use the entryId as the ID of the entry.
       */
      const id = item.entryId;
      /**
       * We need to remove some fields that are not needed in the export.
       */
      delete item.tenant;
      delete item.locale;
      delete item.locked;
      delete item.webinyVersion;
      delete item.version;
      delete item.entryId;
      delete item.modelId;
      const values = item.values;
      delete item.values;
      return {
        ...item,
        ...values,
        id
      };
    }),
    meta,
    after
  }));
};
class CmsEntryZipper {
  constructor(params) {
    this.zipper = params.zipper;
    this.fetcher = params.fetcher;
    this.entryAssets = params.entryAssets;
    this.uniqueAssetsResolver = params.uniqueAssetsResolver;
  }
  async execute(params) {
    const {
      isCloseToTimeout,
      isAborted,
      model,
      after: inputAfter,
      exportAssets
    } = params;
    const files = [];
    let after = inputAfter;
    let hasMoreItems = true;
    let storedFiles = false;
    let id = 1;
    let continueAfter = undefined;
    let assets = undefined;
    /**
     * This function works as self invoking function, it will add items to the zipper until there are no more items to add.
     *
     * If the lambda is close to timeout, we will store the current state and continue from the last cursor in the next task run.
     */
    const addItems = async () => {
      if (isAborted()) {
        this.zipper.abort();
        return;
      }
      const closeToTimeout = isCloseToTimeout();
      if (storedFiles) {
        await this.zipper.finalize();
        return;
      } else if (!hasMoreItems || closeToTimeout) {
        if (closeToTimeout && hasMoreItems) {
          continueAfter = after;
        }
        const output = {
          files,
          assets,
          model: (0, _sanitize.sanitizeModel)({
            id: model.group.id
          }, model)
        };
        await this.zipper.add(Buffer.from(JSON.stringify(output)), {
          name: _constants.MANIFEST_JSON
        });
        storedFiles = true;
        return;
      }
      const {
        items,
        meta
      } = await this.fetcher(after);
      if (meta.totalCount === 0) {
        console.log("No items found, aborting...");
        this.zipper.abort();
        return;
      }
      const name = `entries${inputAfter ? `-${inputAfter}` : ""}-${id}.json`;
      files.push({
        id,
        name,
        after
      });
      await this.zipper.add(createBufferData({
        items,
        meta,
        after
      }), {
        name
      });
      hasMoreItems = meta.hasMoreItems;
      after = meta.cursor || undefined;
      id++;
      /**
       * We should not continue if assets are getting exported.
       * There will be a new task triggered for exporting assets.
       */
      if (exportAssets) {
        return;
      } else if (!assets) {
        assets = [];
      }
      const itemsAssets = await this.entryAssets.assignAssets(items);
      if (itemsAssets.length === 0) {
        return;
      }
      const uniqueItemAssets = this.uniqueAssetsResolver.resolve(itemsAssets, "url");
      if (uniqueItemAssets.length === 0) {
        return;
      }
      assets.push(...uniqueItemAssets);
    };
    this.zipper.on("error", error => {
      console.error(error);
    });
    this.zipper.on("entry", () => {
      addItems();
    });
    addItems();
    const result = await this.zipper.done();
    if (!result.Key) {
      throw new Error("Failed to upload the file.");
    }
    const checksum = (0, _cleanChecksum.cleanChecksum)(result.ETag || "");
    const key = (0, _exportPath.stripExportPath)(result.Key);
    if (continueAfter) {
      return new _CmsEntryZipperExecuteContinueResult.CmsEntryZipperExecuteContinueResult({
        key,
        checksum,
        cursor: continueAfter
      });
    }
    return new _CmsEntryZipperExecuteDoneResult.CmsEntryZipperExecuteDoneResult({
      key,
      checksum
    });
  }
}
exports.CmsEntryZipper = CmsEntryZipper;

//# sourceMappingURL=CmsEntryZipper.js.map